@@ -1,16 +1,10 @@
-#!/usr/bin/env python
-from __future__ import print_function
-
-import io
 import os
-import zipfile
-from shutil import copy2
-
 import lxml
 import time
+import shutil
 import flask
-import logging
 import hashlib
+import logging
 import pandas as pd
 
 try:
@@ -18,27 +12,25 @@ try:
 except:
     import json
 
+from uuid import uuid4
 from urllib.parse import unquote
 from datetime import datetime
 from copy import deepcopy
 from inspect import currentframe, getframeinfo
-from flask import request, jsonify, make_response, Response, Response as HttpResponse, send_file
+from flask import request, jsonify, make_response, Response, Response as HttpResponse, send_file, session, redirect
 from flask_api import status
 
-import label_studio.utils.db as db
 from label_studio.utils.functions import generate_sample_task
-from label_studio.utils.analytics import Analytics
-from label_studio.utils.models import DEFAULT_PROJECT_ID, Project, MLBackend
-from label_studio.utils.prompts import LabelStudioConfigPrompt
-from label_studio.utils.io import find_dir, find_editor_files, get_temp_file
+from label_studio.utils.io import find_dir, find_editor_files, get_temp_dir
 from label_studio.utils import uploader
 from label_studio.utils.validation import TaskValidator
 from label_studio.utils.exceptions import ValidationError
 from label_studio.utils.functions import generate_sample_task_without_check, data_examples
 from label_studio.utils.misc import (
-    exception_treatment, log_config, log, config_line_stripped, config_comments_free,
+    exception_treatment, log_config, log, config_line_stripped,
     get_config_templates, iter_config_templates
 )
+from label_studio.project import Project
 
 logger = logging.getLogger(__name__)
 
@@ -46,65 +38,44 @@ app = flask.Flask(__name__, static_url_path='')
 app.secret_key = 'A0Zrdqwf1AQWj12ajkhgFN]dddd/,?RfDWQQT'
 app.config['SEND_FILE_MAX_AGE_DEFAULT'] = 0
 
-# init
-c = None
-# config file path
-config_path = None
 # input arguments
 input_args = None
-# load editor config from XML
-label_config_line = None
-# analytics
-analytics = None
-# machine learning backend
-ml_backend = None
-# project object with lazy initialization
-project = None
 
 
-def reload_config(prompt_inputs=False, force=False):
-    global c
-    global label_config_line
-    global analytics
-    global ml_backend
-    global project
-    global config_path
+def project_get_or_create(multi_session_force_recreate=False):
+    """
+    Return existed or create new project based on environment. Currently supported methods:
+    - "fixed": project is based on "project_name" attribute specified by input args when app starts
+    - "session": project is based on "project_name" key restored from flask.session object
+    :return:
+    """
+    if input_args.command == 'start-multi-session':
+        # get user from session
+        if 'user' not in session:
+            session['user'] = str(uuid4())
+        user = session['user']
 
-    # Read config from config.json & input arguments
-    c = json.load(open(config_path))
-    c['port'] = input_args.port if input_args.port else c['port']
-    c['label_config'] = input_args.label_config if input_args.label_config else c['label_config']
-    c['input_path'] = input_args.input_path if input_args.input_path else c['input_path']
-    c['output_dir'] = input_args.output_dir if input_args.output_dir else c['output_dir']
-    c['debug'] = input_args.debug if input_args.debug is not None else c['debug']
+        # get project from session
+        if 'project' not in session or multi_session_force_recreate:
+            session['project'] = str(uuid4())
+        project = session['project']
 
-    # If specified, prompt user in console about specific inputs
-    if prompt_inputs:
-        iprompt = LabelStudioConfigPrompt(c)
-        c['input_data'] = iprompt.ask_input_path()
-        c['output_dir'] = iprompt.ask_output_dir()
-        c['label_config'] = iprompt.ask_label_config()
-
-    # Initialize DBs
-    db.re_init(c)
-
-    label_config_full = config_comments_free(open(c['label_config']).read())
-    label_config_line = config_line_stripped(label_config_full)
-    if analytics is None:
-        analytics = Analytics(label_config_line, c.get('collect_analytics', True))
+        project_name = user + '/' + project
+        return Project.get_or_create(project_name, input_args, context={
+            'user': user,
+            'project': project,
+            'multi_session': True,
+        })
     else:
-        analytics.update_info(label_config_line, c.get('collect_analytics', True))
-    # configure project
-    if project is None or force:
-        project = Project(label_config=label_config_line, label_config_full=label_config_full)
-    # configure machine learning backend
-    if ml_backend is None or force:
-        ml_backend_params = c.get('ml_backend')
-        if ml_backend_params:
-            ml_backend = MLBackend.from_params(ml_backend_params)
-            project.connect(ml_backend)
-
-    return True
+        if multi_session_force_recreate:
+            raise NotImplementedError(
+                '"multi_session_force_recreate" option supported only with "start-multi-session" mode')
+        user = project = input_args.project_name  # in standalone mode, user and project are singletons and consts
+        return Project.get_or_create(input_args.project_name, input_args, context={
+            'user': user,
+            'project': project,
+            'multi_session': False
+        })
 
 
 @app.template_filter('json')
@@ -145,104 +116,126 @@ def send_log():
 def labeling_page():
     """ Label studio frontend: task labeling
     """
-    global c
-    global label_config_line
-
-    # reload config at each page reload (for fast changing of config/input_path/output_path)
-    reload_config()
+    project = project_get_or_create()
+    if len(project.tasks) == 0:
+        return redirect('/welcome')
 
     # task data: load task or task with completions if it exists
     task_data = None
     task_id = request.args.get('task_id', None)
 
     if task_id is not None:
-        task_data = db.get_task_with_completions(task_id) or db.get_task(task_id)
-        if ml_backend:
+        task_data = project.get_task_with_completions(task_id) or project.get_task(task_id)
+        if project.ml_backend:
             task_data = deepcopy(task_data)
-            task_data['predictions'] = ml_backend.make_predictions(task_data, project)
+            task_data['predictions'] = project.ml_backend.make_predictions(task_data, project)
 
-    analytics.send(getframeinfo(currentframe()).function)
-    return flask.render_template('labeling.html', config=c, label_config_line=label_config_line,
-                                 task_id=task_id, task_data=task_data, **find_editor_files())
+    project.analytics.send(getframeinfo(currentframe()).function)
+    return flask.render_template(
+        'labeling.html',
+        config=project.config,
+        label_config_line=project.label_config_line,
+        task_id=task_id,
+        task_data=task_data,
+        **find_editor_files()
+    )
 
 
 @app.route('/welcome')
 def welcome_page():
     """ Label studio frontend: task labeling
     """
-    global c, project
-    reload_config()
-    analytics.send(getframeinfo(currentframe()).function)
-    return flask.render_template('welcome.html', config=c, project=project)
+    project = project_get_or_create()
+    project.analytics.send(getframeinfo(currentframe()).function)
+    project.update_on_boarding_state()
+    return flask.render_template(
+        'welcome.html',
+        config=project.config,
+        project=project.project_obj,
+        on_boarding=project.on_boarding
+    )
 
 
-@app.route('/data')
+@app.route('/tasks')
 def tasks_page():
     """ Tasks and completions page: tasks.html
     """
-    global c
-    reload_config()
-    label_config = open(c['label_config']).read()  # load editor config from XML
-    task_ids = db.get_tasks().keys()
-    completed_at = db.get_completed_at(task_ids)
+    project = project_get_or_create()
+
+    label_config = open(project.config['label_config']).read()  # load editor config from XML
+    task_ids = project.get_tasks().keys()
+    completed_at = project.get_completed_at(task_ids)
 
     # sort by completed time
     task_ids = sorted([(i, completed_at[i] if i in completed_at else '9') for i in task_ids], key=lambda x: x[1])
     task_ids = [i[0] for i in task_ids]  # take only id back
-    analytics.send(getframeinfo(currentframe()).function)
-    return flask.render_template('data.html', config=c, label_config=label_config,
-                                 task_ids=task_ids, completions=db.get_completions_ids(),
-                                 completed_at=completed_at)
+    project.analytics.send(getframeinfo(currentframe()).function)
+    return flask.render_template(
+        'tasks.html',
+        show_paths=input_args.command != 'start-multi-session',
+        config=project.config,
+        label_config=label_config,
+        task_ids=task_ids,
+        completions=project.get_completions_ids(),
+        completed_at=completed_at
+    )
 
 
 @app.route('/setup')
 def setup_page():
     """ Setup label config
     """
-    global c, project
-    reload_config()
+    project = project_get_or_create()
 
     templates = get_config_templates()
     input_values = {}
-    analytics.send(getframeinfo(currentframe()).function)
-    return flask.render_template('setup.html', config=c, project=project, templates=templates,
-                                 input_values=input_values)
+    project.analytics.send(getframeinfo(currentframe()).function)
+    return flask.render_template(
+        'setup.html',
+        config=project.config,
+        project=project.project_obj,
+        label_config_full=project.label_config_full,
+        templates=templates,
+        input_values=input_values,
+        multi_session=input_args.command == 'start-multi-session'
+    )
 
 
 @app.route('/import')
 def import_page():
     """ Import tasks from JSON, CSV, ZIP and more
     """
-    global c, project
-    reload_config()
+    project = project_get_or_create()
 
-    analytics.send(getframeinfo(currentframe()).function)
-    return flask.render_template('import.html', config=c, project=project)
+    project.analytics.send(getframeinfo(currentframe()).function)
+    return flask.render_template(
+        'import.html',
+        config=project.config,
+        project=project.project_obj
+    )
 
 
 @app.route('/export')
 def export_page():
     """ Export completions as JSON or using converters
     """
-    global c, project
-    reload_config()
-
-    analytics.send(getframeinfo(currentframe()).function)
-    return flask.render_template('export.html', config=c, project=project)
+    project = project_get_or_create()
+    project.analytics.send(getframeinfo(currentframe()).function)
+    return flask.render_template(
+        'export.html',
+        config=project.config,
+        formats=project.converter.supported_formats,
+        project=project.project_obj
+    )
 
 
 @app.route('/api/render-label-studio', methods=['GET', 'POST'])
 def api_render_label_studio():
     """ Label studio frontend rendering for iframe
     """
-    global c
-    global label_config_line
-
-    # reload config at each page reload (for fast changing of config/input_path/output_path)
-    reload_config()
-
     # get args
-    full_editor = request.args.get('full_editor', False)
+    project = project_get_or_create()
+
     config = request.args.get('config', request.form.get('config', ''))
     config = unquote(config)
     if not config:
@@ -252,12 +245,12 @@ def api_render_label_studio():
     examples = data_examples(mode='editor_preview')
     task_data = {
         data_key: examples.get(data_type, '')
-        for data_key, data_type in Project.extract_data_types(config).items()
+        for data_key, data_type in project.extract_data_types(config).items()
     }
     example_task_data = {
         'id': 1764,
         'data': task_data,
-        'project': DEFAULT_PROJECT_ID,
+        'project': project.id,
         'created_at': '2019-02-06T14:06:42.000420Z',
         'updated_at': '2019-02-06T14:06:42.000420Z'
     }
@@ -269,7 +262,8 @@ def api_render_label_studio():
         'task_ser': example_task_data
     }
     response.update(find_editor_files())
-    analytics.send(getframeinfo(currentframe()).function)
+
+    project.analytics.send(getframeinfo(currentframe()).function)
     return flask.render_template('render_ls.html', **response)
 
 
@@ -277,10 +271,9 @@ def api_render_label_studio():
 def api_validate_config():
     """ Validate label config via tags schema
     """
-    global project
     if 'label_config' not in request.form:
         return make_response('No label_config in POST', status.HTTP_417_EXPECTATION_FAILED)
-
+    project = project_get_or_create()
     try:
         project.validate_label_config(request.form['label_config'])
     except ValidationError as e:
@@ -295,10 +288,10 @@ def api_validate_config():
 def api_save_config():
     """ Save label config
     """
-    global c, project
     if 'label_config' not in request.form:
         return make_response('No label_config in POST', status.HTTP_417_EXPECTATION_FAILED)
 
+    project = project_get_or_create()
     # check config before save
     label_config = request.form['label_config']
     try:
@@ -308,13 +301,9 @@ def api_save_config():
     except Exception as e:
         return make_response(jsonify({'label_config': [str(e)]}), status.HTTP_400_BAD_REQUEST)
 
-    # save xml label config to file
-    path = c['label_config']
-    open(path, 'w').write(label_config)
-    logger.info('Label config saved to: {path}'.format(path=path))
-
-    reload_config(force=True)
-    analytics.send(getframeinfo(currentframe()).function)
+    project.update_label_config(label_config)
+    project.reload()
+    project.analytics.send(getframeinfo(currentframe()).function)
     return Response(status=status.HTTP_201_CREATED)
 
 
@@ -325,12 +314,12 @@ def api_import_example():
     # django compatibility
     request.GET = request.args
     request.POST = request.form
-
+    project = project_get_or_create()
     config = request.GET.get('label_config', '')
     if not config:
         config = request.POST.get('label_config', '')
     try:
-        Project.validate_label_config(config)
+        project.validate_label_config(config)
         output = generate_sample_task_without_check(config, mode='editor_preview')
     except (ValueError, ValidationError, lxml.etree.Error, KeyError):
         response = HttpResponse('error while example generating', status=status.HTTP_400_BAD_REQUEST)
@@ -343,8 +332,7 @@ def api_import_example():
 def api_import_example_file():
     """ Task examples for import
     """
-    global c, project
-    reload_config()
+    project = project_get_or_create()
     request.GET = request.args  # django compatibility
 
     q = request.GET.get('q', 'json')
@@ -387,13 +375,13 @@ def api_import_example_file():
     response.headers['Content-Disposition'] = 'attachment; filename=%s' % filename
     response.headers['filename'] = filename
 
-    analytics.send(getframeinfo(currentframe()).function)
+    project.analytics.send(getframeinfo(currentframe()).function)
     return response
 
 
 @app.route('/api/import', methods=['POST'])
 def api_import():
-    global project, c
+    project = project_get_or_create()
 
     # make django compatibility for uploader module
     class DjangoRequest:
@@ -414,9 +402,9 @@ def api_import():
         return make_response(jsonify(e.msg_to_list()), status.HTTP_400_BAD_REQUEST)
 
     # save task file to input dir
-    if os.path.isdir(c['input_path']):
+    if os.path.isdir(project.config['input_path']):
         # tasks are in directory, write a new file with tasks
-        task_dir = c['input_path']
+        task_dir = project.config['input_path']
         now = datetime.now()
         data = json.dumps(new_tasks, ensure_ascii=False)
         md5 = hashlib.md5(json.dumps(data).encode('utf-8')).hexdigest()
@@ -425,18 +413,18 @@ def api_import():
         tasks = new_tasks
     else:
         # tasks are all in one file, append it
-        path = c['input_path']
+        path = project.config['input_path']
         old_tasks = json.load(open(path))
         assert isinstance(old_tasks, list), 'Tasks from input_path must be list'
         tasks = old_tasks + new_tasks
         logger.error("It's recommended to use directory as input_path: " +
-                     c['input_path'] + ' -> ' + os.path.dirname(c['input_path']))
+                     project.config['input_path'] + ' -> ' + os.path.dirname(project.config['input_path']))
 
     with open(path, 'w') as f:
         json.dump(tasks, f, ensure_ascii=False, indent=4)
 
     # load new tasks
-    db.re_init(c)
+    project.reload()
 
     duration = time.time() - start
     return make_response(jsonify({
@@ -449,49 +437,64 @@ def api_import():
 
 @app.route('/api/export', methods=['GET'])
 def api_export():
-    global c
-
+    export_format = request.args.get('format')
+    project = project_get_or_create()
     now = datetime.now()
-    output_dir = c['output_dir']
-    export_file = now.strftime('%Y-%m-%d-%H-%M-%S') + '.json.export'
-    files = [os.path.join(output_dir, f) for f in os.listdir(output_dir) if f.endswith('.json')]
-    completions = [json.load(open(f)) for f in files]
-    path = os.path.join(output_dir, export_file)
-    json.dump(completions, open(path, 'w'))
-    response = send_file(os.path.abspath(path), as_attachment=True)
-    response.headers['filename'] = export_file
-    return response
+    completion_dir = project.config['output_dir']
+    export_dirname = now.strftime('%Y-%m-%d-%H-%M-%S')
+    with get_temp_dir() as temp_dir:
+        export_dirpath = os.path.join(temp_dir, export_dirname)
+        project.converter.convert(completion_dir, export_dirpath, format=export_format)
+        shutil.make_archive(export_dirpath, 'zip', export_dirpath)
+        export_zipfile = export_dirpath + '.zip'
+        response = send_file(export_zipfile, as_attachment=True)
+        response.headers['filename'] = os.path.basename(export_zipfile)
+        project.analytics.send(getframeinfo(currentframe()).function)
+        return response
 
 
-@app.route('/api/projects/' + str(DEFAULT_PROJECT_ID) + '/next/', methods=['GET'])
+@app.route('/api/projects/1/next/', methods=['GET'])
 @exception_treatment
 def api_generate_next_task():
     """ Generate next task to label
     """
     # try to find task is not presented in completions
-    completions = db.get_completions_ids()
-    for task_id, task in db.iter_tasks():
+    project = project_get_or_create()
+    completions = project.get_completions_ids()
+    for task_id, task in project.iter_tasks():
         if task_id not in completions:
             log.info(msg='New task for labeling', extra=task)
-            analytics.send(getframeinfo(currentframe()).function)
+            project.analytics.send(getframeinfo(currentframe()).function)
             # try to use ml backend for predictions
-            if ml_backend:
+            if project.ml_backend:
                 task = deepcopy(task)
-                task['predictions'] = ml_backend.make_predictions(task, project)
+                task['predictions'] = project.ml_backend.make_predictions(task, project.project_obj)
             return make_response(jsonify(task), 200)
 
     # no tasks found
-    analytics.send(getframeinfo(currentframe()).function, error=404)
+    project.analytics.send(getframeinfo(currentframe()).function, error=404)
     return make_response('', 404)
 
 
-@app.route('/api/projects/' + str(DEFAULT_PROJECT_ID) + '/task_ids/', methods=['GET'])
+@app.route('/api/project/', methods=['POST', 'GET'])
+@exception_treatment
+def api_project():
+    """ Project global operation
+    """
+    project = project_get_or_create(multi_session_force_recreate=False)
+    if request.method == 'POST' and request.args.get('new', False):
+        project = project_get_or_create(multi_session_force_recreate=True)
+    return make_response(jsonify({'project_name': project.name}), 201)
+
+
+@app.route('/api/projects/1/task_ids/', methods=['GET'])
 @exception_treatment
 def api_all_task_ids():
     """ Get all tasks ids
     """
-    ids = sorted(db.get_task_ids())
-    analytics.send(getframeinfo(currentframe()).function)
+    project = project_get_or_create()
+    ids = sorted(project.get_task_ids())
+    project.analytics.send(getframeinfo(currentframe()).function)
     return make_response(jsonify(ids), 200)
 
 
@@ -501,19 +504,31 @@ def api_tasks(task_id):
     """ Get task by id
     """
     # try to get task with completions first
-    task_data = db.get_task_with_completions(task_id)
-    task_data = db.get_task(task_id) if task_data is None else task_data
-    analytics.send(getframeinfo(currentframe()).function)
+    project = project_get_or_create()
+    task_data = project.get_task_with_completions(task_id)
+    task_data = project.get_task(task_id) if task_data is None else task_data
+    project.analytics.send(getframeinfo(currentframe()).function)
     return make_response(jsonify(task_data), 200)
 
 
-@app.route('/api/projects/' + str(DEFAULT_PROJECT_ID) + '/completions_ids/', methods=['GET'])
+@app.route('/api/tasks/delete', methods=['DELETE'])
+@exception_treatment
+def api_tasks_delete():
+    """ Delete all tasks & completions
+    """
+    project = project_get_or_create()
+    project.delete_tasks()
+    return make_response(jsonify({}), 204)
+
+
+@app.route('/api/projects/1/completions_ids/', methods=['GET'])
 @exception_treatment
 def api_all_completion_ids():
     """ Get all completion ids
     """
-    ids = db.get_completions_ids()
-    analytics.send(getframeinfo(currentframe()).function)
+    project = project_get_or_create()
+    ids = project.get_completions_ids()
+    project.analytics.send(getframeinfo(currentframe()).function)
     return make_response(jsonify(ids), 200)
 
 
@@ -522,21 +537,21 @@ def api_all_completion_ids():
 def api_completions(task_id):
     """ Delete or save new completion to output_dir with the same name as task_id
     """
-    global c
+    project = project_get_or_create()
 
     if request.method == 'POST':
         completion = request.json
         completion.pop('state', None)  # remove editor state
-        completion_id = db.save_completion(task_id, completion)
+        completion_id = project.save_completion(task_id, completion)
         log.info(msg='Completion saved', extra={'task_id': task_id, 'output': request.json})
         # try to train model with new completions
-        if ml_backend:
-            ml_backend.update_model(db.get_task(task_id), completion, project)
-        analytics.send(getframeinfo(currentframe()).function)
+        if project.ml_backend:
+            project.ml_backend.update_model(project.get_task(task_id), completion, project.project_obj)
+        project.analytics.send(getframeinfo(currentframe()).function)
         return make_response(json.dumps({'id': completion_id}), 201)
 
     else:
-        analytics.send(getframeinfo(currentframe()).function, error=500)
+        project.analytics.send(getframeinfo(currentframe()).function, error=500)
         return make_response('Incorrect request method', 500)
 
 
@@ -546,18 +561,18 @@ def api_completion_by_id(task_id, completion_id):
     """ Delete or save new completion to output_dir with the same name as task_id.
         completion_id with different IDs is not supported in this backend
     """
-    global c
+    project = project_get_or_create()
 
     if request.method == 'DELETE':
-        if c.get('allow_delete_completions', False):
-            db.delete_completion(task_id)
-            analytics.send(getframeinfo(currentframe()).function)
+        if project.config.get('allow_delete_completions', False):
+            project.delete_completion(task_id)
+            project.analytics.send(getframeinfo(currentframe()).function)
             return make_response('deleted', 204)
         else:
-            analytics.send(getframeinfo(currentframe()).function, error=422)
+            project.analytics.send(getframeinfo(currentframe()).function, error=422)
             return make_response('Completion removing is not allowed in server config', 422)
     else:
-        analytics.send(getframeinfo(currentframe()).function, error=500)
+        project.analytics.send(getframeinfo(currentframe()).function, error=500)
         return make_response('Incorrect request method', 500)
 
 
@@ -567,24 +582,25 @@ def api_completion_update(task_id, completion_id):
     """ Rewrite existing completion with patch.
         This is technical api call for editor testing only. It's used for Rewrite button in editor.
     """
-    global c
+    project = project_get_or_create()
     completion = request.json
 
     completion.pop('state', None)  # remove editor state
     completion['id'] = int(completion_id)
-    db.save_completion(task_id, completion)
+    project.save_completion(task_id, completion)
     log.info(msg='Completion saved', extra={'task_id': task_id, 'output': request.json})
-    analytics.send(getframeinfo(currentframe()).function)
+    project.analytics.send(getframeinfo(currentframe()).function)
     return make_response('ok', 201)
 
 
-@app.route('/api/projects/' + str(DEFAULT_PROJECT_ID) + '/expert_instruction')
+@app.route('/api/projects/1/expert_instruction')
 @exception_treatment
 def api_instruction():
     """ Instruction for annotators
     """
-    analytics.send(getframeinfo(currentframe()).function)
-    return make_response(c['instruction'], 200)
+    project = project_get_or_create()
+    project.analytics.send(getframeinfo(currentframe()).function)
+    return make_response(project.config['instruction'], 200)
 
 
 @app.route('/predict', methods=['POST'])
@@ -593,12 +609,13 @@ def api_predict():
     """ Make ML prediction using ml_backend
     """
     task = request.json
+    project = project_get_or_create()
     if project.ml_backend:
-        predictions = project.ml_backend.make_predictions({'data': task}, project)
-        analytics.send(getframeinfo(currentframe()).function)
+        predictions = project.ml_backend.make_predictions({'data': task}, project.project_obj)
+        project.analytics.send(getframeinfo(currentframe()).function)
         return make_response(jsonify(predictions), 200)
     else:
-        analytics.send(getframeinfo(currentframe()).function, error=400)
+        project.analytics.send(getframeinfo(currentframe()).function, error=400)
         return make_response(jsonify("No ML backend"), 400)
 
 
@@ -610,98 +627,6 @@ def get_data_file(filename):
     return flask.send_from_directory(directory, filename, as_attachment=True)
 
 
-def label_studio_init(output_dir, label_config=None):
-    os.makedirs(output_dir, exist_ok=True)
-    default_config_file = os.path.join(output_dir, 'config.json')
-    default_label_config_file = os.path.join(output_dir, 'config.xml')
-    default_output_dir = os.path.join(output_dir, 'completions')
-    default_input_path = os.path.join(output_dir, 'tasks.json')
-
-    if label_config:
-        copy2(label_config, default_label_config_file)
-
-    default_config = {
-        'title': 'Label Studio',
-        'port': 8200,
-        'debug': False,
-
-        'label_config': default_label_config_file,
-        'input_path': default_input_path,
-        'output_dir': default_output_dir,
-
-        'instruction': 'Type some <b>hypertext</b> for label experts!',
-        'allow_delete_completions': True,
-        'templates_dir': 'examples',
-
-        'editor': {
-            'debug': False
-        },
-
-        '!ml_backend': {
-            'url': 'http://localhost:9090',
-            'model_name': 'my_super_model'
-        },
-        'sampling': 'uniform'
-    }
-
-    # create input_path (tasks.json)
-    if not os.path.exists(default_input_path):
-        with io.open(default_input_path, mode='w') as fout:
-            json.dump([], fout, indent=2)
-        print(default_input_path + ' input path has been created.')
-    else:
-        print(default_input_path +' input path already exists.')
-
-    # create config file (config.json)
-    if not os.path.exists(default_config_file):
-        with io.open(default_config_file, mode='w') as fout:
-            json.dump(default_config, fout, indent=2)
-        print(default_config_file + ' config file has been created.')
-    else:
-        print(default_config_file + ' config file already exists.')
-
-    # create label config (config.xml)
-    if not os.path.exists(default_label_config_file):
-        default_label_config = '<View></View>'
-        with io.open(default_label_config_file, mode='w') as fout:
-            fout.write(default_label_config)
-        print(default_label_config_file + ' label config file has been created.')
-    else:
-        print(default_label_config_file + ' label config file already exists.')
-
-    # create output dir (completions)
-    if not os.path.exists(default_output_dir):
-        os.makedirs(default_output_dir)
-        print(default_output_dir + ' output directory has been created.')
-    else:
-        print(default_output_dir + ' output directory already exists.')
-
-    print('')
-    print('Label Studio has been successfully initialized. Check project states in ' + output_dir)
-    print('Start the server: label-studio start ' + output_dir)
-
-
-def _get_config(args):
-    # if config is explicitly specified, just return it
-    if args.config_path:
-        return args.config_path
-
-    # check if project directory exists
-    if not os.path.exists(args.project_name):
-        raise FileNotFoundError(
-            'Couldn\'t find directory ' + args.project_name + ', maybe you\'ve missed appending "--init" option:\n'
-            'label-studio start ' + args.project_name + ' --init')
-
-    # check config.json exists in directory
-    config_path = os.path.join(args.project_name, 'config.json')
-    if not os.path.exists(config_path):
-        raise FileNotFoundError(
-            'Couldn\'t find config file ' + config_path + ' in project directory ' + args.project_name + ', '
-            'maybe you\'ve missed appending "--init" option:\nlabel-studio start ' + args.project_name + ' --init')
-
-    return config_path
-
-
 def parse_input_args():
     """ Combine args with json config
 
@@ -722,6 +647,13 @@ def parse_input_args():
         '-d', '--debug', dest='debug', action='store_true',
         help='Debug mode for Flask', default=None
     )
+    root_parser.add_argument(
+        '--root-dir', dest='root_dir', default='.',
+        help='Projects root directory'
+    )
+    root_parser.add_argument(
+        '-v', '--verbose', dest='verbose', action='store_true',
+        help='Increase output verbosity')
 
     parser = argparse.ArgumentParser(description='Label studio')
 
@@ -772,8 +704,41 @@ def parse_input_args():
         '-p', '--port', dest='port', default=8200, type=int,
         help='Server port')
     parser_start.add_argument(
-        '-v', '--verbose', action='store_true',
-        help='Increase output verbosity')
+        '--ml-backend-url', dest='ml_backend_url',
+        help='Machine learning backend URL')
+    parser_start.add_argument(
+        '--ml-backend-name', dest='ml_backend_name',
+        help='Machine learning backend name')
+
+    # start-multi-session sub-command parser
+
+    parser_start_ms = subparsers.add_parser(
+        'start-multi-session', help='Start Label Studio server', parents=[root_parser])
+    parser_start_ms.add_argument(
+        '--template', dest='template', choices=available_templates,
+        help='Choose from predefined project templates'
+    )
+    parser_start_ms.add_argument(
+        '-c', '--config', dest='config_path',
+        help='Server config')
+    parser_start_ms.add_argument(
+        '-l', '--label-config', dest='label_config', default='',
+        help='Label config path')
+    parser_start_ms.add_argument(
+        '-i', '--input-path', dest='input_path', default='',
+        help='Input path to task file or directory with tasks')
+    parser_start_ms.add_argument(
+        '-o', '--output-dir', dest='output_dir', default='',
+        help='Output directory for completions')
+    parser_start_ms.add_argument(
+        '-p', '--port', dest='port', default=8200, type=int,
+        help='Server port')
+    parser_start_ms.add_argument(
+        '--ml-backend-url', dest='ml_backend_url',
+        help='Machine learning backend URL')
+    parser_start_ms.add_argument(
+        '--ml-backend-name', dest='ml_backend_name',
+        help='Machine learning backend name')
 
     args = parser.parse_args()
     label_config_explicitly_specified = hasattr(args, 'label_config') and args.label_config
@@ -781,37 +746,44 @@ def parse_input_args():
         args.label_config = os.path.join(find_dir('examples'), args.template, 'config.xml')
     if not hasattr(args, 'label_config'):
         args.label_config = None
-
     return args
 
 
 def main():
     import threading
     import webbrowser
-    global config_path, input_args
-    import sys
+
+    import label_studio.utils.functions
+
+    global input_args
 
     input_args = parse_input_args()
 
+    label_studio.utils.functions.HOSTNAME = 'http://localhost:' + str(input_args.port)
+
     # On `init` command, create directory args.project_name with initial project state and exit
     if input_args.command == 'init':
-        label_studio_init(input_args.project_name, input_args.label_config)
+        Project.create_project_dir(input_args.project_name, input_args)
         return
 
-    # If `start --init` option is specified, do the same as with `init` command, but continue to run app
-    elif input_args.init:
-        label_studio_init(input_args.project_name, input_args.label_config)
+    elif input_args.command == 'start':
+
+        # If `start --init` option is specified, do the same as with `init` command, but continue to run app
+        if input_args.init:
+            Project.create_project_dir(input_args.project_name, input_args)
 
     # On `start` command, launch browser if --no-browser is not specified and start label studio server
     if input_args.command == 'start':
-        config_path = _get_config(input_args)
-        reload_config()
         if not input_args.no_browser:
-            browser_url = 'http://127.0.0.1:' + str(c["port"]) + '/welcome'
+            browser_url = 'http://127.0.0.1:' + str(input_args.port) + '/welcome'
             threading.Timer(2.5, lambda: webbrowser.open(browser_url)).start()
             print('Start browser at URL: ' + browser_url)
 
-        app.run(host='0.0.0.0', port=c['port'], debug=c['debug'])
+        app.run(host='0.0.0.0', port=input_args.port, debug=input_args.debug)
+
+    # On `start-multi-session` command, server creates one project per each browser sessions
+    elif input_args.command == 'start-multi-session':
+        app.run(host='0.0.0.0', port=input_args.port, debug=input_args.debug)
 
 
 if __name__ == "__main__":
