@@ -1,8 +1,12 @@
 import os
 import pkg_resources
+import shutil
+import glob
 
 from contextlib import contextmanager
-from tempfile import mkstemp
+from tempfile import mkstemp, mkdtemp
+
+from appdirs import user_config_dir, user_data_dir
 
 
 def find_node(package_name, node_path, node_type):
@@ -54,3 +58,29 @@ def get_temp_file():
     fd, path = mkstemp()
     yield path
     os.close(fd)
+
+
+@contextmanager
+def get_temp_dir():
+    dirpath = mkdtemp()
+    yield dirpath
+    shutil.rmtree(dirpath)
+
+
+def get_config_dir():
+    config_dir = user_config_dir(appname='label-studio')
+    if not os.path.exists(config_dir):
+        os.makedirs(config_dir)
+    return config_dir
+
+
+def get_data_dir():
+    data_dir = user_data_dir(appname='label-studio')
+    if not os.path.exists(data_dir):
+        os.makedirs(data_dir)
+    return data_dir
+
+
+def delete_dir_content(dirpath):
+    for f in glob.glob(dirpath + '/*'):
+        os.remove(f)
